package Model;

// These operators form a compositional translation from the
// calculus into some specific model.

public abstract class Operators
{ // All of these operators may be destructive.

  // Some models support constants as well as purenames. These
  // models should treat the 'dat' arguments in Snd, Rcv and Fus
  // as follows. If the first char is an integer, then it's an
  // integer. If it's ", then it's a string. If the dat is true
  // or false then it's a boolean. Otherwise it's presumably a name.

  // Par -- constructs P|Q
  public abstract State Par(State P,State Q) throws Exception;
  // Sum -- constructs P+Q. Not all models implement this. 
  public abstract State Sum(State P,State Q) throws Exception;
  // Res -- construct (ns)P.
  public abstract State Res(String[] ns,State P) throws Exception;
  // Rep -- constructs !P. Some models have restrictions on Res.
  public abstract State Rep(State P) throws Exception;
  // Snd -- constructs n!d.P. Each dat may be an integer, or "string literal", or boolean, or name.
  public abstract State Snd(String n,String[] dat,boolean isbound,State P) throws Exception;
  // Rcv -- as for Snd
  public abstract State Rcv(String n,String[] dat,boolean isbound,State P) throws Exception;
  // Nil -- constructs 0
  public abstract State Nil() throws Exception;
  // Fus -- constructs d1=d2. 
  public abstract State Fus(String dat1,String dat2) throws Exception;

  public State ParAccumulate(State s1,State s2) throws Exception
  { if (s1!=null && s2!=null) return Par(s1,s2);
    if (s1!=null) return s1; else return s2;
  }
  public State SumAccumulate(State s1,State s2) throws Exception
  { if (s1!=null && s2!=null) return Sum(s1,s2);
    if (s1!=null) return s1; else return s2;
  }

}
